/*
Smart Door

  Smart Door is an easy solution to connect your door to the smartphone with a few easy steps.
  Smart Door will notify you when you forgot to lock the door and when someone is approaching your door.

  link to video:
  Created By:
  Eilon Goldstein 203641972
  Idan Ashkenazi 313294985 
*/ 

// Libraries
#include "ESP8266WiFi.h"
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"
#include "BlynkSimpleEsp8266.h"

// Wifi Settings
#define WLAN_SSID       "*****" //ENTER WIFI NAME
#define WLAN_PASS       "*****" //ENTER WIFI PASSWORD

// Adafruit IO
#define AIO_SERVER      "io.adafruit.com"
#define AIO_SERVERPORT  1883
#define AIO_USERNAME    "*****" //ENTER Adafruit USERNAME
#define AIO_KEY         "*****" //ENTER Adafruit KEY

//Blynk token
#define BLYNK_AUTH "*****" //ENTER Blynk AUTHENTICATION KEY

// ESP8266 pin tied to trigger pin on the ultrasonic sensor.
#define TRIGGER_PIN D8  
// ESP8266 pin tied to echo pin on the ultrasonic sensor.
#define ECHO_PIN D7
// maximum distance to expect object
#define MAX_DISTANCE 100

// Notes
#define NOTE_C4 523
#define NOTE_FS4 370
const int alertNotes[] = {NOTE_C4, NOTE_FS4, NOTE_C4};
const int speakerPin = D6; // digital output pin that the Speaker is attached to

// potentiometer
const int potentiometer = A0;
int potentiometerValue = 0;

// Create an ESP8266 WiFiClient class to connect to the MQTT server.
WiFiClient client;

// Store the MQTT server, client ID, username, and password in flash memory.
const char MQTT_SERVER[] = AIO_SERVER;

// Set a unique MQTT client ID using the AIO key + the date and time the sketch
// was compiled (so this should be unique across multiple devices for a user,
// alternatively you can manually set this to a GUID or other random value).
const char MQTT_CLIENTID[] = AIO_KEY __DATE__ __TIME__;
const char MQTT_USERNAME[] = AIO_USERNAME;
const char MQTT_PASSWORD[] = AIO_KEY;

// Setup the MQTT client class by passing in the WiFi client and MQTT server and login details.
Adafruit_MQTT_Client mqtt(&client, MQTT_SERVER, AIO_SERVERPORT, MQTT_CLIENTID, MQTT_USERNAME, MQTT_PASSWORD);/****************************** Feeds ***************************************/

// Setup feeds 
const char KEY_FEED_POTENTIOMETWR[] = AIO_USERNAME "/feeds/potentiometer";
Adafruit_MQTT_Publish keyPotentiometer = Adafruit_MQTT_Publish(&mqtt, KEY_FEED_POTENTIOMETWR);

const char KEY_FEED_ULTRASONIC[] = AIO_USERNAME "/feeds/ultrasonic";
Adafruit_MQTT_Publish keyUltrasonic = Adafruit_MQTT_Publish(&mqtt, KEY_FEED_ULTRASONIC);

/*************************** Sketch Code ************************************/

void setup() {

  Serial.begin(115200);
  delay(10);
  Serial.println("Adafruit IO Example");
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WLAN_SSID);

  WiFi.begin(WLAN_SSID, WLAN_PASS);
  // wait for WiFi connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    delay(5);
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println();
  // connect to adafruit io
  connect();
  // connect to Blynk
  Blynk.begin(BLYNK_AUTH, WLAN_SSID, WLAN_PASS);
}

void loop() {
  Blynk.run();
  pingAdafruit();
  boolean isLock = isDoorLock();
  int cm = ultrasonic_measure();
  publishData(cm, isLock);
  
  if (cm < 30 && isLock) {
    Blynk.notify("Someone is at the door!");
    delay(1000);
    alert();
  }
  else if(!isLock){
    Blynk.notify("You forgot to lock the door!!");
  }
  delay(10000);
}

void pingAdafruit(){
    // ping adafruit io a few times to make sure we remain connected
  if (!mqtt.ping(3)) {
    // reconnect to adafruit io
    if (!mqtt.connected())
      connect();
  }
}

// connect to adafruit io via MQTT
void connect() {
  Serial.print("Connecting to Adafruit IO... ");
  int8_t ret;
  while ((ret = mqtt.connect()) != 0) {
    switch (ret) {
      case 1: Serial.println("Wrong protocol"); break;
      case 2: Serial.println("ID rejected"); break;
      case 3: Serial.println("Server unavail"); break;
      case 4: Serial.println("Bad user/pass"); break;
      case 5: Serial.println("Not authed"); break;
      case 6: Serial.println("Failed to subscribe"); break;
      default: Serial.println("Connection failed"); break;
    }
    if (ret >= 0) {
      mqtt.disconnect();
    }
    Serial.println("Retrying connection...");
    delay(5000);
  }
  Serial.println("Adafruit IO Connected!");
}

void publishData(int cm, boolean isLock){
  if (!keyUltrasonic.publish(cm))
    Serial.println("Failed to publish ultrasonic measurement");
  else
    Serial.println("ultrasonic measurement published!");
  if (!keyPotentiometer.publish(isLock))
    Serial.println("Failed to publish Lock status");
  else
    Serial.println("Lock status published!");
}

boolean isDoorLock(){
  potentiometerValue = analogRead(potentiometer);
  potentiometerValue = map(potentiometerValue, 0, 1023, 0, 9);
  potentiometerValue = constrain(potentiometerValue, 0, 9);
  Serial.println(potentiometerValue);
  boolean isLock = (potentiometerValue > 3);
  Serial.println(isLock);
  return isLock;
}

int ultrasonic_measure() {
  pinMode(TRIGGER_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
  digitalWrite(TRIGGER_PIN, LOW);
  delayMicroseconds(2);
  digitalWrite(TRIGGER_PIN, HIGH);
  delayMicroseconds(5);
  digitalWrite(TRIGGER_PIN, LOW);
  int duration = pulseIn(ECHO_PIN, HIGH, MAX_DISTANCE * 59);
  int cm = duration / 59;
  Serial.println(cm);
  return cm;
}

void alert(){
  int size = sizeof(alertNotes)/sizeof(alertNotes[0]);
  for(int j = 0; j < 3; j++) {
    for(int i = 0; i < size; i++) {
      tone(speakerPin, alertNotes[i]);
      delay(400);
      noTone(speakerPin);
    }
    delay(600);
  }
}
